#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Possible CGI entry point for the owl package.

@author: U{Ivan Herman<a href="http://www.w3.org/People/Ivan/">}
@license: This software is available for use under the
U{W3C® SOFTWARE NOTICE AND LICENSE<href="http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231">}
@contact: Ivan Herman, ivan@w3.org
"""

"""
$Id: RDFConvertService.py,v 1.3 2009/07/10 08:45:14 ivan Exp $
"""

__version__ = "4.0"

import sys
import cgi
import cgitb
cgitb.enable()
form = cgi.FieldStorage()
from owlrl import convert_graph, RDFXML, TURTLE, AUTO

# ---------------------------------------------------------------------------------------------------------------------


class Options:
    def __init__(self, frm):
        self.iformat = AUTO
        self.owlClosure = "no"
        self.rdfsClosure = "no"
        self.owlExtras = "no"
        self.axioms = "no"
        self.daxioms = "no"
        self.sources = []
        self.text = None
        self.format = TURTLE

        if "source_1" in list(frm.keys()):
            self.sources.append(frm["source_1"].value)
        if "source_2" in list(frm.keys()):
            self.sources.append(frm["source_2"].value)
        
        if "text" in list(frm.keys()):
            self.text = frm["text"].value
        if "format" in list(frm.keys()):
            self.format = frm["format"].value
        if "iformat" in list(frm.keys()):
            v = frm["iformat"].value
            if v == "xml" or v == "turtle":
                self.iformat = v
            
        if "fullClosure" in list(frm.keys()) and frm["fullClosure"].value == "yes":
            self.owlClosure = "yes"
            self.rdfsClosure = "yes"
            self.axioms = "no"
            self.daxioms = "no"
            self.owlExtras = "no"
        else:
            if "owlClosure" in list(frm.keys()):
                self.owlClosure = frm["owlClosure"].value
            if "rdfsClosure" in list(frm.keys()):
                self.rdfsClosure = frm["rdfsClosure"].value
            if "owlExtras" in list(frm.keys()):
                self.owlExtras = frm["owlExtras"].value
            if "axioms" in list(frm.keys()):
                self.axioms = frm["axioms"].value
            if "daxioms" in list(frm.keys()):
                self.daxioms = frm["daxioms"].value

        # this one is for backward compatibility...
        if "uri" in list(frm.keys()):
            self.sources.append(frm["uri"].value)
        if "source" in list(frm.keys()):
            self.sources.append(frm["source"].value)
        
    def to_html(self):
        print('<dl>')
        
        print('<dt>Sources:</dt>')
        print('<dd>')
        if len(self.sources) == 0:
            print("none")
        elif len(self.sources) == 1:
            print(cgi.escape(self.sources[0]))
        else:
            print(cgi.escape(self.sources[0]), ", ", cgi.escape(self.sources[1]))
        print('</dd>')
        
        print('<dt>Input format:</dt>')
        print('<dd>%s</dd>' % self.iformat)
        print('<dt>Output format:</dt><dd>%s</dd>' % self.format)
        print('<dt>OWL 2 RL Processing:</dt><dd>%s</dd>' % self.owlClosure)
        print('<dt>RDFS Processing:</dt><dd>%s</dd>' % self.rdfsClosure)
        print('<dt>Extra OWL Processing:</dt><dd>%s</dd>' % self.owlExtras)
        print('<dt>Axiomatic triples added:</dt><dd>%s</dd>' % self.axioms)
        print('<dt>Datatype Axiomatic triples added:</dt><dd>%s</dd>' % self.daxioms)
        if self.text is not None:
            print('<dt>Turtle code added to the graph:</dt>')
        print('<dl>')
        if self.text is not None:
            print(cgi.escape(self.text).replace('\n', '<br/>'))

# ---------------------------------------------------------------------------------------------------------------------


options = Options(form)

try:
    retval = convert_graph(options)
    if options.format == TURTLE:
        print('Content-Type: text/turtle; charset=utf-8')
    else:
        print('Content-Type: application/rdf+xml; charset=utf-8')
    print()
    print(retval)
except:
    (typ, value, traceback) = sys.exc_info()
    print('Content-type: text/html; charset=utf-8')
    print('Status: 400 Invalid Input')
    print()
    print("<html>")
    print("<head>")
    print("<title>Error in RDF Closure processing</title>")
    print("</head><body>")
    print("<h1>Error in RDF Closure processing:</h1>")
    print("<pre>%s</pre>" % value)
    print("<h1>For reference, the input arguments were:</h1>")
    options.to_html()
    print("</body>")
    print("</html>")



#!/d/Bin/Python/python.exe
# -*- coding: utf-8 -*-
#
"""
Axiomatic triples to be (possibly) added to the final graph.

**Requires**: `RDFLib`_, 4.0.0 and higher.

.. _RDFLib: https://github.com/RDFLib/rdflib

**License**: This software is available for use under the `W3C Software License`_.

.. _W3C Software License: http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231

**Organization**: `World Wide Web Consortium`_

.. _World Wide Web Consortium: http://www.w3.org

**Author**: `Ivan Herman`_

.. _Ivan Herman: http://www.w3.org/People/Ivan/

"""

__author__  = 'Ivan Herman'
__contact__ = 'Ivan Herman, ivan@w3.org'
__license__ = 'W3C® SOFTWARE NOTICE AND LICENSE, http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231'

import rdflib
from owlrl.RDFS import Seq, Bag, Alt, Statement, Property, XMLLiteral, List
from owlrl.RDFS import RDFNS as ns_rdf
from owlrl.RDFS import rdf_subject, rdf_predicate, rdf_object, rdf_type, value, first, rest, nil
from owlrl.RDFS import Resource, Class, subClassOf, subPropertyOf, comment, label, rdfs_domain, rdfs_range
from owlrl.RDFS import seeAlso, isDefinedBy, Literal, Container, ContainerMembershipProperty, member, Datatype

from rdflib.namespace     import XSD as ns_xsd
from .OWL import *

# Simple RDF axiomatic triples (typing of subject, predicate, first, rest, etc)
_Simple_RDF_axiomatic_triples = [
    (rdf_type, rdf_type, Property),
    (rdf_subject, rdf_type, Property),
    (rdf_predicate, rdf_type, Property),
    (rdf_object, rdf_type, Property),
    (first, rdf_type, Property),
    (rest, rdf_type, Property),
    (value, rdf_type, Property),
    (nil, rdf_type, List),
]

# RDFS axiomatic triples (domain and range, as well as class setting for a number of RDFS symbols)
_RDFS_axiomatic_triples = [
    (rdf_type, rdfs_domain, Resource),
    (rdfs_domain, rdfs_domain, Property),
    (rdfs_range, rdfs_domain, Property),
    (subPropertyOf, rdfs_domain, Property),
    (subClassOf, rdfs_domain, Class),
    (rdf_subject, rdfs_domain, Statement),
    (rdf_predicate, rdfs_domain, Statement),
    (rdf_object, rdfs_domain, Statement),
    (member, rdfs_domain, Resource),
    (first, rdfs_domain, List),
    (rest, rdfs_domain, List),
    (seeAlso, rdfs_domain, Resource),
    (isDefinedBy, rdfs_domain, Resource),
    (comment, rdfs_domain, Resource),
    (label, rdfs_domain, Resource),
    (value, rdfs_domain, Resource),
    (Property, rdf_type, Class),

    (rdf_type, rdfs_range, Class),
    (rdfs_domain, rdfs_range, Class),
    (rdfs_range, rdfs_range, Class),
    (subPropertyOf, rdfs_range, Property),
    (subClassOf, rdfs_range, Class),
    (rdf_subject, rdfs_range, Resource),
    (rdf_predicate, rdfs_range, Resource),
    (rdf_object, rdfs_range, Resource),
    (member, rdfs_range, Resource),
    (first, rdfs_range, Resource),
    (rest, rdfs_range, List),
    (seeAlso, rdfs_range, Resource),
    (isDefinedBy, rdfs_range, Resource),
    (comment, rdfs_range, Literal),
    (label, rdfs_range, Literal),
    (value, rdfs_range, Resource),

    (Alt, subClassOf, Container),
    (Bag, subClassOf, Container),
    (Seq, subClassOf, Container),
    (ContainerMembershipProperty, subClassOf, Property),

    (isDefinedBy, subPropertyOf, seeAlso),

    (XMLLiteral, rdf_type, Datatype),
    (XMLLiteral, subClassOf, Literal),
    (Datatype, subClassOf, Class),

    # rdfs valid triples; these would be inferred by the RDFS expansion, but it may make things
    # a bit faster to add these upfront
    (Resource, rdf_type, Class),
    (Class, rdf_type, Class),
    (Literal, rdf_type, Class),
    (XMLLiteral, rdf_type, Class),
    (Datatype, rdf_type, Class),
    (Seq, rdf_type, Class),
    (Bag, rdf_type, Class),
    (Alt, rdf_type, Class),
    (Container, rdf_type, Class),
    (List, rdf_type, Class),
    (ContainerMembershipProperty, rdf_type, Class),
    (Property, rdf_type, Class),
    (Statement, rdf_type, Class),

    (rdfs_domain, rdf_type, Property),
    (rdfs_range, rdf_type, Property),
    (subPropertyOf, rdf_type, Property),
    (subClassOf, rdf_type, Property),
    (member, rdf_type, Property),
    (seeAlso, rdf_type, Property),
    (isDefinedBy, rdf_type, Property),
    (comment, rdf_type, Property),
    (label, rdf_type, Property)
]

#: RDFS Axiomatic Triples all together
RDFS_Axiomatic_Triples    = _Simple_RDF_axiomatic_triples + _RDFS_axiomatic_triples

#: RDFS D-entailement triples, ie, possible subclassing of various datatypes
RDFS_D_Axiomatic_Triples_subclasses = [
    # See http://www.w3.org/TR/2004/REC-xmlschema-2-20041028/#built-in-datatypes
    (ns_xsd['decimal'], subClassOf, Literal),

    (ns_xsd['integer'], subClassOf, ns_xsd['decimal']),

    (ns_xsd['long'], subClassOf, ns_xsd['integer']),
    (ns_xsd['int'], subClassOf, ns_xsd['long']),
    (ns_xsd['short'], subClassOf, ns_xsd['int']),
    (ns_xsd['byte'], subClassOf, ns_xsd['short']),

    (ns_xsd['nonNegativeInteger'], subClassOf, ns_xsd['integer']),
    (ns_xsd['positiveInteger'], subClassOf, ns_xsd['nonNegativeInteger']),
    (ns_xsd['unsignedLong'], subClassOf, ns_xsd['nonNegativeInteger']),
    (ns_xsd['unsignedInt'], subClassOf, ns_xsd['unsignedLong']),
    (ns_xsd['unsignedShort'], subClassOf, ns_xsd['unsignedInt']),
    (ns_xsd['unsignedByte'], subClassOf, ns_xsd['unsignedShort']),

    (ns_xsd['nonPositiveInteger'], subClassOf, ns_xsd['integer']),
    (ns_xsd['negativeInteger'], subClassOf, ns_xsd['nonPositiveInteger']),

    (ns_xsd['normalizedString'], subClassOf, ns_xsd['string']),
    (ns_xsd['token'], subClassOf, ns_xsd['normalizedString']),
    (ns_xsd['language'], subClassOf, ns_xsd['token']),
    (ns_xsd['Name'], subClassOf, ns_xsd['token']),
    (ns_xsd['NMTOKEN'], subClassOf, ns_xsd['token']),

    (ns_xsd['NCName'], subClassOf, ns_xsd['Name']),

    (ns_xsd['dateTimeStamp'], subClassOf, ns_xsd['dateTime']),
]

#:
RDFS_D_Axiomatic_Triples_types = [
    (ns_xsd['integer'], rdf_type, Datatype),
    (ns_xsd['decimal'], rdf_type, Datatype),
    (ns_xsd['nonPositiveInteger'], rdf_type, Datatype),
    (ns_xsd['nonPositiveInteger'], rdf_type, Datatype),
    (ns_xsd['positiveInteger'], rdf_type, Datatype),
    (ns_xsd['positiveInteger'], rdf_type, Datatype),
    (ns_xsd['long'], rdf_type, Datatype),
    (ns_xsd['int'], rdf_type, Datatype),
    (ns_xsd['short'], rdf_type, Datatype),
    (ns_xsd['byte'], rdf_type, Datatype),
    (ns_xsd['unsignedLong'], rdf_type, Datatype),
    (ns_xsd['unsignedInt'], rdf_type, Datatype),
    (ns_xsd['unsignedShort'], rdf_type, Datatype),
    (ns_xsd['unsignedByte'], rdf_type, Datatype),
    (ns_xsd['float'], rdf_type, Datatype),
    (ns_xsd['double'], rdf_type, Datatype),
    (ns_xsd['string'], rdf_type, Datatype),
    (ns_xsd['normalizedString'], rdf_type, Datatype),
    (ns_xsd['token'], rdf_type, Datatype),
    (ns_xsd['language'], rdf_type, Datatype),
    (ns_xsd['Name'], rdf_type, Datatype),
    (ns_xsd['NCName'], rdf_type, Datatype),
    (ns_xsd['NMTOKEN'], rdf_type, Datatype),
    (ns_xsd['boolean'], rdf_type, Datatype),
    (ns_xsd['hexBinary'], rdf_type, Datatype),
    (ns_xsd['base64Binary'], rdf_type, Datatype),
    (ns_xsd['anyURI'], rdf_type, Datatype),
    (ns_xsd['dateTimeStamp'], rdf_type, Datatype),
    (ns_xsd['dateTime'], rdf_type, Datatype),
    (Literal, rdf_type, Datatype),
    (XMLLiteral, rdf_type, Datatype),
]

RDFS_D_Axiomatic_Triples = RDFS_D_Axiomatic_Triples_types + RDFS_D_Axiomatic_Triples_subclasses

# OWL Class axiomatic triples: definition of special classes
_OWL_axiomatic_triples_Classes = [
    (AllDifferent, rdf_type, Class),
    (AllDifferent, subClassOf, Resource),

    (AllDisjointClasses, rdf_type, Class),
    (AllDisjointClasses, subClassOf, Resource),

    (AllDisjointProperties, rdf_type, Class),
    (AllDisjointProperties, subClassOf, Resource),

    (Annotation, rdf_type, Class),
    (Annotation, subClassOf, Resource),

    (AnnotationProperty, rdf_type, Class),
    (AnnotationProperty, subClassOf, Property),

    (AsymmetricProperty, rdf_type, Class),
    (AsymmetricProperty, subClassOf, Property),

    (OWLClass, rdf_type, Class),
    (OWLClass, equivalentClass, Class),

#    (DataRange, type, Class),
#    (DataRange, equivalentClass, Datatype),

    (Datatype, rdf_type, Class),

    (DatatypeProperty, rdf_type, Class),
    (DatatypeProperty, subClassOf, Property),

    (DeprecatedClass, rdf_type, Class),
    (DeprecatedClass, subClassOf, Class),

    (DeprecatedProperty, rdf_type, Class),
    (DeprecatedProperty, subClassOf, Property),

    (FunctionalProperty, rdf_type, Class),
    (FunctionalProperty, subClassOf, Property),

    (InverseFunctionalProperty, rdf_type, Class),
    (InverseFunctionalProperty, subClassOf, Property),

    (IrreflexiveProperty, rdf_type, Class),
    (IrreflexiveProperty, subClassOf, Property),

    (Literal, rdf_type, Datatype),

#    (NamedIndividual, type, Class),
#    (NamedIndividual, equivalentClass, Resource),

    (NegativePropertyAssertion, rdf_type, Class),
    (NegativePropertyAssertion, subClassOf, Resource),

    (Nothing, rdf_type, Class),
    (Nothing, subClassOf, Thing ),

    (ObjectProperty, rdf_type, Class),
    (ObjectProperty, equivalentClass, Property),

    (Ontology, rdf_type, Class),
    (Ontology, subClassOf, Resource),

    (OntologyProperty, rdf_type, Class),
    (OntologyProperty, subClassOf, Property),

    (Property, rdf_type, Class),

    (ReflexiveProperty, rdf_type, Class),
    (ReflexiveProperty, subClassOf, Property),

    (Restriction, rdf_type, Class),
    (Restriction, subClassOf, Class),


    (SymmetricProperty, rdf_type, Class),
    (SymmetricProperty, subClassOf, Property),

    (Thing, rdf_type, Class),
    (Thing, subClassOf, Resource),

    (TransitiveProperty, rdf_type, Class),
    (TransitiveProperty, subClassOf, Property),

    # OWL valid triples; some of these would be inferred by the OWL RL expansion, but it may make things
    # a bit faster to add these upfront
    (AllDisjointProperties, rdf_type, OWLClass),
    (AllDisjointClasses, rdf_type, OWLClass),
    (AllDisjointProperties, rdf_type, OWLClass),
    (Annotation, rdf_type, OWLClass),
    (AsymmetricProperty, rdf_type, OWLClass),
    (Axiom, rdf_type, OWLClass),
    (DataRange, rdf_type, OWLClass),
    (Datatype, rdf_type, OWLClass),
    (DatatypeProperty, rdf_type, OWLClass),
    (DeprecatedClass, rdf_type, OWLClass),
    (DeprecatedClass, subClassOf, OWLClass),
    (DeprecatedProperty, rdf_type, OWLClass),
    (FunctionalProperty, rdf_type, OWLClass),
    (InverseFunctionalProperty, rdf_type, OWLClass),
    (IrreflexiveProperty, rdf_type, OWLClass),
    (NamedIndividual, rdf_type, OWLClass),
    (NegativePropertyAssertion, rdf_type, OWLClass),
    (Nothing, rdf_type, OWLClass),
    (ObjectProperty, rdf_type, OWLClass),
    (Ontology, rdf_type, OWLClass),
    (OntologyProperty, rdf_type, OWLClass),
    (Property, rdf_type, OWLClass),
    (ReflexiveProperty, rdf_type, OWLClass),
    (Restriction, rdf_type, OWLClass),
    (Restriction, subClassOf, OWLClass),
#    (SelfRestriction, type, OWLClass),
    (SymmetricProperty, rdf_type, OWLClass),
    (Thing, rdf_type, OWLClass),
    (TransitiveProperty, rdf_type, OWLClass),
]

#: OWL Property axiomatic triples: definition of domains and ranges
_OWL_axiomatic_triples_Properties = [
    (allValuesFrom, rdf_type, Property),
    (allValuesFrom, rdfs_domain, Restriction),
    (allValuesFrom, rdfs_range, Class),

    (assertionProperty, rdf_type, Property),
    (assertionProperty, rdfs_domain, NegativePropertyAssertion),
    (assertionProperty, rdfs_range, Property),

    (backwardCompatibleWith, rdf_type, OntologyProperty),
    (backwardCompatibleWith, rdf_type, AnnotationProperty),
    (backwardCompatibleWith, rdfs_domain, Ontology),
    (backwardCompatibleWith, rdfs_range, Ontology),

#    (bottomDataProperty, type, DatatypeProperty),

#    (bottomObjectProperty, type, ObjectProperty),

#    (cardinality, type, Property),
#    (cardinality, domain, Restriction),
#    (cardinality, range, ns_xsd["nonNegativeInteger"]),

    (comment, rdf_type, AnnotationProperty),
    (comment, rdfs_domain, Resource),
    (comment, rdfs_range, Literal),

    (complementOf, rdf_type, Property),
    (complementOf, rdfs_domain, Class),
    (complementOf, rdfs_range, Class),

#    (datatypeComplementOf, type, Property),
#    (datatypeComplementOf, domain, Datatype),
#    (datatypeComplementOf, range, Datatype),

    (deprecated, rdf_type, AnnotationProperty),
    (deprecated, rdfs_domain, Resource),
    (deprecated, rdfs_range, Resource),

    (differentFrom, rdf_type, Property),
    (differentFrom, rdfs_domain, Resource),
    (differentFrom, rdfs_range, Resource),

#    (disjointUnionOf, type, Property),
#    (disjointUnionOf, domain, Class),
#    (disjointUnionOf, range, List),

    (disjointWith, rdf_type, Property),
    (disjointWith, rdfs_domain, Class),
    (disjointWith, rdfs_range, Class),

    (distinctMembers, rdf_type, Property),
    (distinctMembers, rdfs_domain, AllDifferent),
    (distinctMembers, rdfs_range, List),

    (equivalentClass, rdf_type, Property),
    (equivalentClass, rdfs_domain, Class),
    (equivalentClass, rdfs_range, Class),

    (equivalentProperty, rdf_type, Property),
    (equivalentProperty, rdfs_domain, Property),
    (equivalentProperty, rdfs_range, Property),

    (hasKey, rdf_type, Property),
    (hasKey, rdfs_domain, Class),
    (hasKey, rdfs_range, List),

    (hasValue, rdf_type, Property),
    (hasValue, rdfs_domain, Restriction),
    (hasValue, rdfs_range, Resource),

    (imports, rdf_type, OntologyProperty),
    (imports, rdfs_domain, Ontology),
    (imports, rdfs_range, Ontology),

    (incompatibleWith, rdf_type, OntologyProperty),
    (incompatibleWith, rdf_type, AnnotationProperty),
    (incompatibleWith, rdfs_domain, Ontology),
    (incompatibleWith, rdfs_range, Ontology),

    (intersectionOf, rdf_type, Property),
    (intersectionOf, rdfs_domain, Class),
    (intersectionOf, rdfs_range, List),

    (inverseOf, rdf_type, Property),
    (inverseOf, rdfs_domain, Property),
    (inverseOf, rdfs_range, Property),

    (isDefinedBy, rdf_type, AnnotationProperty),
    (isDefinedBy, rdfs_domain, Resource),
    (isDefinedBy, rdfs_range, Resource),

    (label, rdf_type, AnnotationProperty),
    (label, rdfs_domain, Resource),
    (label, rdfs_range, Literal),

    (maxCardinality, rdf_type, Property),
    (maxCardinality, rdfs_domain, Restriction),
    (maxCardinality, rdfs_range, ns_xsd["nonNegativeInteger"]),

    (maxQualifiedCardinality, rdf_type, Property),
    (maxQualifiedCardinality, rdfs_domain, Restriction),
    (maxQualifiedCardinality, rdfs_range, ns_xsd["nonNegativeInteger"]),

    (members, rdf_type, Property),
    (members, rdfs_domain, Resource),
    (members, rdfs_range, List),

#    (minCardinality, type, Property),
#    (minCardinality, domain, Restriction),
#    (minCardinality, range, ns_xsd["nonNegativeInteger"]),

#    (minQualifiedCardinality, type, Property),
#    (minQualifiedCardinality, domain, Restriction),
#    (minQualifiedCardinality, range, ns_xsd["nonNegativeInteger"]),

#    (annotatedTarget, type, Property),
#    (annotatedTarget, domain, Resource),
#    (annotatedTarget, range, Resource),

    (onClass, rdf_type, Property),
    (onClass, rdfs_domain, Restriction),
    (onClass, rdfs_range, Class),

#    (onDataRange, type, Property),
#    (onDataRange, domain, Restriction),
#    (onDataRange, range, Datatype),

    (onDatatype, rdf_type, Property),
    (onDatatype, rdfs_domain, Datatype),
    (onDatatype, rdfs_range, Datatype),

    (oneOf, rdf_type, Property),
    (oneOf, rdfs_domain, Class),
    (oneOf, rdfs_range, List),

    (onProperty, rdf_type, Property),
    (onProperty, rdfs_domain, Restriction),
    (onProperty, rdfs_range, Property),

#    (onProperties, type, Property),
#    (onProperties, domain, Restriction),
#    (onProperties, range, List),

#    (annotatedProperty, type, Property),
#    (annotatedProperty, domain, Resource),
#    (annotatedProperty, range, Property),

    (priorVersion, rdf_type, OntologyProperty),
    (priorVersion, rdf_type, AnnotationProperty),
    (priorVersion, rdfs_domain, Ontology),
    (priorVersion, rdfs_range, Ontology),

    (propertyChainAxiom, rdf_type, Property),
    (propertyChainAxiom, rdfs_domain, Property),
    (propertyChainAxiom, rdfs_range, List),

#    (propertyDisjointWith, type, Property),
#    (propertyDisjointWith, domain, Property),
#    (propertyDisjointWith, range, Property),
#
#    (qualifiedCardinality, type, Property),
#    (qualifiedCardinality, domain, Restriction),
#    (qualifiedCardinality, range, ns_xsd["nonNegativeInteger"]),

    (sameAs, rdf_type, Property),
    (sameAs, rdfs_domain, Resource),
    (sameAs, rdfs_range, Resource),

    (seeAlso, rdf_type, AnnotationProperty),
    (seeAlso, rdfs_domain, Resource),
    (seeAlso, rdfs_range, Resource),

    (someValuesFrom, rdf_type, Property),
    (someValuesFrom, rdfs_domain, Restriction),
    (someValuesFrom, rdfs_range, Class),

    (sourceIndividual, rdf_type, Property),
    (sourceIndividual, rdfs_domain, NegativePropertyAssertion),
    (sourceIndividual, rdfs_range, Resource),
#
#    (annotatedSource, type, Property),
#    (annotatedSource, domain, Resource),
#    (annotatedSource, range, Resource),
#
    (targetIndividual, rdf_type, Property),
    (targetIndividual, rdfs_domain, NegativePropertyAssertion),
    (targetIndividual, rdfs_range, Resource),

    (targetValue, rdf_type, Property),
    (targetValue, rdfs_domain, NegativePropertyAssertion),
    (targetValue, rdfs_range, Literal),

#    (topDataProperty, type, DatatypeProperty),
#    (topDataProperty, domain, Resource),
#    (topDataProperty, range, Literal),
#
#    (topObjectProperty, type, ObjectProperty),
#    (topObjectProperty, domain, Resource),
#    (topObjectProperty, range, Resource),

    (unionOf, rdf_type, Property),
    (unionOf, rdfs_domain, Class),
    (unionOf, rdfs_range, List),

    (versionInfo, rdf_type, AnnotationProperty),
    (versionInfo, rdfs_domain, Resource),
    (versionInfo, rdfs_range, Resource),

    (versionIRI, rdf_type, AnnotationProperty),
    (versionIRI, rdfs_domain, Resource),
    (versionIRI, rdfs_range, Resource),

    (withRestrictions, rdf_type, Property),
    (withRestrictions, rdfs_domain, Datatype),
    (withRestrictions, rdfs_range, List),

    # some OWL valid triples; these would be inferred by the OWL RL expansion, but it may make things
    # a bit faster to add these upfront
    (allValuesFrom, rdfs_range, OWLClass),
    (complementOf, rdfs_domain, OWLClass),
    (complementOf, rdfs_range, OWLClass),

#    (datatypeComplementOf, domain, DataRange),
#    (datatypeComplementOf, range, DataRange),
    (disjointUnionOf, rdfs_domain, OWLClass),
    (disjointWith, rdfs_domain, OWLClass),
    (disjointWith, rdfs_range, OWLClass),
    (equivalentClass, rdfs_domain, OWLClass),
    (equivalentClass, rdfs_range, OWLClass),
    (hasKey, rdfs_domain, OWLClass),
    (intersectionOf, rdfs_domain, OWLClass),
    (onClass, rdfs_range, OWLClass),
#    (onDataRange, range, DataRange),
    (onDatatype, rdfs_domain, DataRange),
    (onDatatype, rdfs_range, DataRange),
    (oneOf, rdfs_domain, OWLClass),
    (someValuesFrom, rdfs_range, OWLClass),
    (unionOf, rdfs_range, OWLClass),
#    (withRestrictions, domain, DataRange)
]

#: OWL RL axiomatic triples: combination of the RDFS triples plus the OWL specific ones
OWLRL_Axiomatic_Triples   = _OWL_axiomatic_triples_Classes   + _OWL_axiomatic_triples_Properties

# Note that this is not used anywhere. But I encoded it once and I did not want to remove it...:-)
_OWL_axiomatic_triples_Facets = [
    # langPattern
    (ns_xsd['length'], rdf_type, Property),
    (ns_xsd['maxExclusive'], rdf_type, Property),
    (ns_xsd['maxInclusive'], rdf_type, Property),
    (ns_xsd['maxLength'], rdf_type, Property),
    (ns_xsd['minExclusive'], rdf_type, Property),
    (ns_xsd['minInclusive'], rdf_type, Property),
    (ns_xsd['minLength'], rdf_type, Property),
    (ns_xsd['pattern'], rdf_type, Property),

    (ns_xsd['length'], rdfs_domain, Resource),
    (ns_xsd['maxExclusive'], rdfs_domain, Resource),
    (ns_xsd['maxInclusive'], rdfs_domain, Resource),
    (ns_xsd['maxLength'], rdfs_domain, Resource),
    (ns_xsd['minExclusive'], rdfs_domain, Resource),
    (ns_xsd['minInclusive'], rdfs_domain, Resource),
    (ns_xsd['minLength'], rdfs_domain, Resource),
    (ns_xsd['pattern'], rdfs_domain, Resource),
    (ns_xsd['length'], rdfs_domain, Resource),

    (ns_xsd['maxExclusive'], rdfs_range, Literal),
    (ns_xsd['maxInclusive'], rdfs_range, Literal),
    (ns_xsd['maxLength'], rdfs_range, Literal),
    (ns_xsd['minExclusive'], rdfs_range, Literal),
    (ns_xsd['minInclusive'], rdfs_range, Literal),
    (ns_xsd['minLength'], rdfs_range, Literal),
    (ns_xsd['pattern'], rdfs_range, Literal),
]

#: OWL D-entailment triples (additionally to the RDFS ones), ie, possible subclassing of various extra datatypes
_OWL_D_Axiomatic_Triples_types = [
    (ns_rdf['PlainLiteral'], rdf_type, Datatype)
]

#:
OWL_D_Axiomatic_Triples_subclasses = [
    (ns_xsd['string'], subClassOf, ns_rdf['PlainLiteral']),
    (ns_xsd['normalizedString'], subClassOf, ns_rdf['PlainLiteral']),
    (ns_xsd['token'], subClassOf, ns_rdf['PlainLiteral']),
    (ns_xsd['Name'], subClassOf, ns_rdf['PlainLiteral']),
    (ns_xsd['NCName'], subClassOf, ns_rdf['PlainLiteral']),
    (ns_xsd['NMTOKEN'], subClassOf, ns_rdf['PlainLiteral'])
]

#:
OWLRL_Datatypes_Disjointness = [
    (ns_xsd["anyURI"], disjointWith, ns_xsd['base64Binary']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['boolean']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['dateTime']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['decimal']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['double']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['float']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["anyURI"], disjointWith, ns_xsd['string']),
    (ns_xsd["anyURI"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["anyURI"], disjointWith, XMLLiteral),

    (ns_xsd["base64Binary"], disjointWith, ns_xsd['boolean']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['dateTime']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['decimal']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['double']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['float']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["base64Binary"], disjointWith, ns_xsd['string']),
    (ns_xsd["base64Binary"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["base64Binary"], disjointWith, XMLLiteral),

    (ns_xsd["boolean"], disjointWith, ns_xsd['dateTime']),
    (ns_xsd["boolean"], disjointWith, ns_xsd['decimal']),
    (ns_xsd["boolean"], disjointWith, ns_xsd['double']),
    (ns_xsd["boolean"], disjointWith, ns_xsd['float']),
    (ns_xsd["boolean"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["boolean"], disjointWith, ns_xsd['string']),
    (ns_xsd["boolean"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["boolean"], disjointWith, XMLLiteral),

    (ns_xsd["dateTime"], disjointWith, ns_xsd['decimal']),
    (ns_xsd["dateTime"], disjointWith, ns_xsd['double']),
    (ns_xsd["dateTime"], disjointWith, ns_xsd['float']),
    (ns_xsd["dateTime"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["dateTime"], disjointWith, ns_xsd['string']),
    (ns_xsd["dateTime"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["dateTime"], disjointWith, XMLLiteral),

    (ns_xsd["decimal"], disjointWith, ns_xsd['double']),
    (ns_xsd["decimal"], disjointWith, ns_xsd['float']),
    (ns_xsd["decimal"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["decimal"], disjointWith, ns_xsd['string']),
    (ns_xsd["decimal"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["decimal"], disjointWith, XMLLiteral),

    (ns_xsd["double"], disjointWith, ns_xsd['float']),
    (ns_xsd["double"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["double"], disjointWith, ns_xsd['string']),
    (ns_xsd["double"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["double"], disjointWith, XMLLiteral),

    (ns_xsd["float"], disjointWith, ns_xsd['hexBinary']),
    (ns_xsd["float"], disjointWith, ns_xsd['string']),
    (ns_xsd["float"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["float"], disjointWith, XMLLiteral),

    (ns_xsd["hexBinary"], disjointWith, ns_xsd['string']),
    (ns_xsd["hexBinary"], disjointWith, ns_rdf['PlainLiteral']),
    (ns_xsd["hexBinary"], disjointWith, XMLLiteral),

    (ns_xsd["string"], disjointWith, XMLLiteral),
]

#: OWL RL D Axiomatic triples: combination of the RDFS ones, plus some extra statements on ranges and domains, plus
#: some OWL specific datatypes
OWLRL_D_Axiomatic_Triples = RDFS_D_Axiomatic_Triples + _OWL_D_Axiomatic_Triples_types + \
                            OWL_D_Axiomatic_Triples_subclasses + OWLRL_Datatypes_Disjointness






"""
OWL and OWL2 terms. Note that the set of terms is *complete*, I.e., it includes *all* OWL 2 terms, regardless of
whether the term is used in OWL 2 RL or not.

**Requires**: `RDFLib`_, 4.0.0 and higher.

.. _RDFLib: https://github.com/RDFLib/rdflib

**License**: This software is available for use under the `W3C Software License`_.

.. _W3C Software License: http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231

**Organization**: `World Wide Web Consortium`_

.. _World Wide Web Consortium: http://www.w3.org

**Author**: `Ivan Herman`_

.. _Ivan Herman: http://www.w3.org/People/Ivan/
"""


import rdflib
from rdflib	import Namespace


# The OWL namespace as used for RDFLib
OWLNS = Namespace("http://www.w3.org/2002/07/owl#")

annotatedSource = OWLNS["annotatedSource"]
annotatedTarget = OWLNS["annotatedTarget"]
annotatedProperty = OWLNS["annotatedProperty"]
allValuesFrom = OWLNS["allValuesFrom"]
assertionProperty = OWLNS["assertionProperty"]
backwardCompatibleWith = OWLNS["backwardCompatibleWith"]
cardinality = OWLNS["cardinality"]
complementOf = OWLNS["complementOf"]
BottomDataProperty = OWLNS["BottomDataProperty"]
BottomObjectProperty = OWLNS["BottomObjectProperty"]
datatypeComplementOf = OWLNS["datatypeComplementOf"]
deprecated = OWLNS["deprecated"]
differentFrom = OWLNS["differentFrom"]
disjointUnionOf = OWLNS["disjointUnionOf"]
disjointClasses = OWLNS["disjointClasses"]
disjointWith = OWLNS["disjointWith"]
distinctMembers = OWLNS["distinctMembers"]
equivalentClass = OWLNS["equivalentClass"]
equivalentProperty = OWLNS["equivalentProperty"]
hasKey = OWLNS["hasKey"]
hasValue = OWLNS["hasValue"]
hasSelf = OWLNS["hasSelf"]
imports = OWLNS["imports"]
incompatibleWith = OWLNS["incompatibleWith"]
intersectionOf = OWLNS["intersectionOf"]
inverseOf = OWLNS["inverseOf"]
maxCardinality = OWLNS["maxCardinality"]
maxQualifiedCardinality = OWLNS["maxQualifiedCardinality"]
members = OWLNS["members"]
minCardinality = OWLNS["minCardinality"]
minQualifiedCardinality = OWLNS["minQualifiedCardinality"]
onClass = OWLNS["onClass"]
onDataRange = OWLNS["onDataRange"]
onDatatype = OWLNS["onDatatype"]
oneOf = OWLNS["oneOf"]
onProperty = OWLNS["onProperty"]
onProperties = OWLNS["onProperties"]
OWLpredicate = OWLNS["predicate"]
priorVersion = OWLNS["priorVersion"]
propertyChainAxiom = OWLNS["propertyChainAxiom"]
propertyDisjointWith = OWLNS["propertyDisjointWith"]
qualifiedCardinality = OWLNS["qualifiedCardinality"]
sameAs = OWLNS["sameAs"]
someValuesFrom = OWLNS["someValuesFrom"]
sourceIndividual = OWLNS["sourceIndividual"]
OWLsubject = OWLNS["subject"]
targetIndividual = OWLNS["targetIndividual"]
targetValue = OWLNS["targetValue"]
TopDataProperty = OWLNS["TopDataProperty"]
TopObjectProperty = OWLNS["TopObjectProperty"]
unionOf = OWLNS["unionOf"]
versionInfo = OWLNS["versionInfo"]
versionIRI = OWLNS["versionIRI"]
withRestrictions = OWLNS["withRestrictions"]

AllDisjointProperties = OWLNS["AllDisjointProperties"]
AllDifferent = OWLNS["AllDifferent"]
AllDisjointClasses = OWLNS["AllDisjointClasses"]
Annotation = OWLNS["Annotation"]
AnnotationProperty = OWLNS["AnnotationProperty"]
AsymmetricProperty = OWLNS["AsymmetricProperty"]
Axiom = OWLNS["Axiom"]
OWLClass = OWLNS["Class"]
DataRange = OWLNS["DataRange"]
DatatypeProperty = OWLNS["DatatypeProperty"]
DeprecatedClass = OWLNS["DeprecatedClass"]
DeprecatedProperty = OWLNS["DeprecatedProperty"]
FunctionalProperty = OWLNS["FunctionalProperty"]
InverseFunctionalProperty = OWLNS["InverseFunctionalProperty"]
IrreflexiveProperty = OWLNS["IrreflexiveProperty"]
NamedIndividual = OWLNS["NamedIndividual"]
NegativePropertyAssertion = OWLNS["NegativePropertyAssertion"]
Nothing = OWLNS["Nothing"]
ObjectProperty = OWLNS["ObjectProperty"]
Ontology = OWLNS["Ontology"]
OntologyProperty = OWLNS["OntologyProperty"]
ReflexiveProperty = OWLNS["ReflexiveProperty"]
Restriction = OWLNS["Restriction"]
Thing = OWLNS["Thing"]
SelfRestriction = OWLNS["SelfRestriction"]
SymmetricProperty = OWLNS["SymmetricProperty"]
TransitiveProperty = OWLNS["TransitiveProperty"]



#!/d/Bin/Python/python.exe
# -*- coding: utf-8 -*-
#
"""
Lists of XSD datatypes and their mutual relationships

**Requires**: `RDFLib`_, 4.0.0 and higher.

.. _RDFLib: https://github.com/RDFLib/rdflib

**License**: This software is available for use under the `W3C Software License`_.

.. _W3C Software License: http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231

**Organization**: `World Wide Web Consortium`_

.. _World Wide Web Consortium: http://www.w3.org

**Author**: `Ivan Herman`_

.. _Ivan Herman: http://www.w3.org/People/Ivan/

"""
__author__ = 'Ivan Herman'
__contact__ = 'Ivan Herman, ivan@w3.org'
__license__ = 'W3C® SOFTWARE NOTICE AND LICENSE, http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231'

# noinspection PyPep8Naming
from .RDFS import RDFNS as ns_rdf
from .RDFS import Literal
from .RDFS import XMLLiteral
from .RDFS import HTMLLiteral
from .RDFS import LangString

import rdflib
# noinspection PyPep8Naming
from rdflib.namespace import XSD as ns_xsd

#: The basic XSD types used everywhere; this means not the complete set of day/month types
_Common_XSD_Datatypes = [
    ns_xsd['integer'], ns_xsd['decimal'], ns_xsd['nonNegativeInteger'], ns_xsd['nonPositiveInteger'],
    ns_xsd['negativeInteger'], ns_xsd['positiveInteger'], ns_xsd['long'], ns_xsd['int'], ns_xsd['short'],
    ns_xsd['byte'], ns_xsd['unsignedLong'], ns_xsd['unsignedInt'], ns_xsd['unsignedShort'],
    ns_xsd['unsignedByte'], ns_xsd['float'], ns_xsd['double'], ns_xsd['string'], ns_xsd['normalizedString'],
    ns_xsd['token'], ns_xsd['language'], ns_xsd['Name'], ns_xsd['NCName'], ns_xsd['NMTOKEN'],
    ns_xsd['boolean'], ns_xsd['hexBinary'], ns_xsd['base64Binary'], ns_xsd['anyURI'],
    ns_xsd['dateTimeStamp'], ns_xsd['dateTime'], ns_xsd['time'], ns_xsd['date'],
    Literal, XMLLiteral, HTMLLiteral, LangString
]

#: RDFS Datatypes: the basic ones plus the complete set of day/month ones
RDFS_Datatypes = _Common_XSD_Datatypes + [ns_xsd['gYearMonth'], ns_xsd['gMonthDay'], ns_xsd['gYear'],
                                          ns_xsd['gDay'], ns_xsd['gMonth']]

#: OWL RL Datatypes: the basic ones plus plain literal
OWL_RL_Datatypes = _Common_XSD_Datatypes + [ns_rdf['PlainLiteral']]

#: XSD Datatype subsumptions
_Common_Datatype_Subsumptions = {
    ns_xsd['dateTimeStamp']: [ns_xsd['dateTime']],
    ns_xsd['integer']: [ns_xsd['decimal']],
    ns_xsd['long']: [ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['int']: [ns_xsd['long'], ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['short']: [ns_xsd['int'], ns_xsd['long'], ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['byte']: [ns_xsd['short'], ns_xsd['int'], ns_xsd['long'], ns_xsd['integer'], ns_xsd['decimal']],

    ns_xsd['nonNegativeInteger']: [ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['positiveInteger']: [ns_xsd['nonNegativeInteger'], ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['unsignedLong']: [ns_xsd['nonNegativeInteger'], ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['unsignedInt']: [ns_xsd['unsignedLong'], ns_xsd['nonNegativeInteger'], ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['unsignedShort']: [ns_xsd['unsignedInt'], ns_xsd['unsignedLong'], ns_xsd['nonNegativeInteger'],
                              ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['unsignedByte']: [ns_xsd['unsignedShort'], ns_xsd['unsignedInt'], ns_xsd['unsignedLong'],
                             ns_xsd['nonNegativeInteger'], ns_xsd['integer'], ns_xsd['decimal']],

    ns_xsd['nonPositiveInteger']: [ns_xsd['integer'], ns_xsd['decimal']],
    ns_xsd['negativeInteger']: [ns_xsd['nonPositiveInteger'], ns_xsd['integer'], ns_xsd['decimal']],

    ns_xsd['normalizedString']: [ns_xsd["string"]],
    ns_xsd['token']: [ns_xsd['normalizedString'], ns_xsd["string"]],
    ns_xsd['language']: [ns_xsd['token'], ns_xsd['normalizedString'], ns_xsd["string"]],
    ns_xsd['Name']: [ns_xsd['token'], ns_xsd['normalizedString'], ns_xsd["string"]],
    ns_xsd['NCName']: [ns_xsd['Name'], ns_xsd['token'], ns_xsd['normalizedString'], ns_xsd["string"]],
    ns_xsd['NMTOKEN']: [ns_xsd['Name'], ns_xsd['token'], ns_xsd['normalizedString'], ns_xsd["string"]],
}

#: RDFS Datatype subsumptions: at the moment, there is no extra to XSD
RDFS_Datatype_Subsumptions = _Common_Datatype_Subsumptions

#: OWL Datatype subsumptions: at the moment, there is no extra to XSD
OWL_Datatype_Subsumptions = _Common_Datatype_Subsumptions



#!/usr/bin/env python3
#
from optparse import OptionParser
from owlrl import convert_graph, RDFXML, TURTLE, JSON, AUTO, RDFA


def main():
    parser = OptionParser(usage="%prog [options] fname1 fname2 ...")
    parser.disable_interspersed_args()
    
    # The 'text' field is not used in the command line, but the CGI environment uses it. This means that there
    # is no option to change that, but is added to the final option structure
    parser.set_defaults(format=TURTLE, owlClosure="no", rdfsClosure="no", owlExtras="no", axioms="no", daxioms="no",
                        iformat=AUTO, trimming="no", maximal="no", text=None)
    
    parser.add_option("-f", "--file", type="string", dest="source",
                      help="input file; should be a .rdf or .ttl file, for RDF/XML or Turtle, respectively. If "
                            "missing, or if the value is '-' then standard input is used. Usage of this options is not "
                            "really necessary, the fname in the command lines refer to files by themselves")
    
    parser.add_option("--owlrl", action="store", dest="owlClosure", choices=["yes", "no"],
                      help="execute OWL RL closure; argument must be yes|no [default: %default]")
    
    parser.add_option("-w", action="store_const", dest="owlClosure", const="yes",
                      help="OWL-RL is executed; shorthand for --owlrl=yes")
    
    parser.add_option("--rdfs", action="store", dest="rdfsClosure", choices=["yes", "no"],
                      help="execute RDFS closure; argument must be yes|no [default: %default]")

    parser.add_option("-r", action="store_const", dest="rdfsClosure", const="yes",
                      help="RDFS is executed; shorthand for --rdfs=yes")

    parser.add_option("--extras", action="store", dest="owlExtras", choices=["yes", "no"],
                      help="additional owl features added; argument must be yes|no [default: %default]")

    parser.add_option("-e", action="store_const", dest="owlExtras", const="yes",
                      help="additional owl features added; shorthand for --extras=yes [default: %default]")

    parser.add_option("--axioms", action="store", dest="axioms", choices=["yes", "no"],
                      help="axiomatic triples; argument must be yes|no [default: %default]")
    
    parser.add_option("-a", action="store_const", dest="axioms", const="yes",
                      help="add axiomatic triples; shorthand for --axioms=yes [default: %default]")

    parser.add_option("--daxioms", action="store", dest="daxioms", choices=["yes", "no"],
                      help="datatype axiomatic triples; argument must be true|false [default: %default]")
    
    parser.add_option("-d", action="store_const", dest="daxioms", const="yes",
                      help="add axiomatic triples; shorthand for --daxioms=yes [default: %default]")

    parser.add_option("--trimming", action="store", dest="trimming", choices=["yes", "no"],
                      help="trim the output of OWL 2 RL and extension; 'yes' is ineffective unless --extras=yes  "
                      "[default: %default]")

    parser.add_option("-m", "--maximal", action="store_const", dest="maximal", const="yes",
                      help="maximal possibilities switched on: RDFS, OWL with extensions, and with trimming")

    parser.add_option("-t", action="store_const", dest="trimming", const="yes",
                      help="trim the output of OWL 2 RL and extension; shorthand for --trimming=yes "
                           "[default: %default]")
    
    parser.add_option("-o", "-s", "--serialization", "--syntax", action="store", dest="format",
                      choices=[TURTLE, JSON, RDFXML],
                      help="output format; argument must be turtle|json|xml [default: %default]")
    
    parser.add_option("-i", "--input_syntax", action="store", dest="iformat",
                      choices=[AUTO, TURTLE, JSON, RDFA, RDFXML],
                      help="format of input; argument must be auto|turtle|xml|rdfa|json [default: %default]; auto "
                            "means that file suffix defines the format. This flag is valid for all input files.")

    (options, args) = parser.parse_args()
    if options.source is None:
        options.sources = []
    else:
        options.sources = [options.source]
        
    if len(args) > 0:
        options.sources += args
        
    if len(options.sources) == 0:
        # the default mechanism, ie, to use standard input
        options.sources = ["-"]

    if options.maximal == "yes":
        options.trimming = "yes"
        options.owlClosure = "yes"
        options.owlExtras = "yes"
            
    print(convert_graph(options))


# The standard startup idiom...
if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
#
"""
Separate module to handle literals. 

The issue is that pure literals cannot appear in subject position according to the current rules on RDF. That means that
different types of conclusions cannot properly finish. The present trick is trying to get around the problem as follows:

#. For all literals in the graph a bnode is created. The module does not do a full D entailment but just relies on RDFLib's ability to recognize identical literals
#. All those bnodes get a type Literal
#. All triples with literals are exchanged against a triple with the associated bnode

The inferences are then calculated with the modified Graph. At the end of the process, the above steps are done
backwards: for all triples where a bnode representing a literal appear in object position, a triple is generated;
however, all triples where the bnode appears in a subject position are removed from the final graph.


**Requires**: `RDFLib`_, 4.0.0 and higher.

.. _RDFLib: https://github.com/RDFLib/rdflib

**License**: This software is available for use under the `W3C Software License`_.

.. _W3C Software License: http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231

**Organization**: `World Wide Web Consortium`_

.. _World Wide Web Consortium: http://www.w3.org

**Author**: `Ivan Herman`_

.. _Ivan Herman: http://www.w3.org/People/Ivan/
"""
__author__ = 'Ivan Herman'
__contact__ = 'Ivan Herman, ivan@w3.org'
__license__ = 'W3C® SOFTWARE NOTICE AND LICENSE, http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231'

from functools import partial

from rdflib import BNode
from rdflib import Literal as rdflibLiteral
from rdflib.namespace import XSD as ns_xsd

from .RDFS import rdf_type
from .RDFS import Literal
from .DatatypeHandling import AltXSDToPYTHON


identity = lambda v: v


class _LiteralStructure:
    """
    This class serves as a wrapper around rdflib's Literal, by changing the equality function to a strict
    identity of datatypes and lexical values.
    
    On the other hand, to implement, eg, OWL RL's datatype rules, a should be able to generate 
    an 'a sameAs b' triple, ie, the distinction should be kept. Hence this class that overrides the equality,
    and then can be used as a key in a Python dictionary.
    """

    # noinspection PyPep8
    def __init__(self, lit):
        self.lit = lit
        self.lex = str(lit)
        self.dt = lit.datatype
        self.lang = lit.language
        self.value = lit.value

#    def compare_value(self, other ):
#        """Compare to literal structure instances for equality. Here equality means in the sense of datatype values
#        @return: comparison result
#        @rtype: boolean
#        """
#        try:
#            if self.dt == OWLNS["rational"] and other.dt == OWLNS["rational"]:
#                return self.value == other.value
#            elif self.dt == OWLNS["rational"] and other.dt != OWLNS["rational"]:
#                l = rdflibLiteral(float(self.lit._cmp_value))
#                return l == other.lit
#            elif self.dt != OWLNS["rational"] and other.dt == OWLNS["rational"]:
#                l = rdflibLiteral(float(other.lit._cmp_value))
#                return self.lit == l
#            else:
#                return self.lit == other.lit
#        except: 
#            # There might be conversion problems...
#            return False

    # noinspection PyBroadException
    def compare_value(self, other):
        """Compare to literal structure instances for equality. Here equality means in the sense of datatype values
        @return: comparison result
        @rtype: boolean
        """
        try:
            return self.lit == other.lit
        except: 
            # There might be conversion problems...
            return False

    def __eq__(self, other):
        if other is None:
            return False
        else:
            return self.lex == other.lex and self.dt == other.dt and self.lang == other.lang

    def __ne__(self, other):
        return not self.__eq__(other)
        
    def __hash__(self):
        if self.dt is not None:
            return hash(self.lit) ^ hash(self.dt)
        else:
            return hash(self.lit)
            
    def __repr__(self):
        retval = ""
        retval += "lexical value: %s; " % self.lex
        retval += "datatype: %s; " % self.dt
        retval += "language tag: %s; " % self.lang
        return retval


class LiteralProxies:
    # noinspection PyPep8
    """

    :param graph: The graph to be modified.
    :param closure: ..
    """
    def __init__(self, graph, closure):
        """
        @param graph: the graph to be modified
        """
        self.lit_to_bnode = {}
        self.bnode_to_lit = {}
        self.graph = graph
        
        to_be_added = set()
        _add_bnode = partial(add_bnode, self, to_be_added)

        # This is supposed to be a "proper" graph, so get the triples which
        # object is a literal. All of them will be removed from graph and
        # replaced with the ones stored in `to_be_added` variable.
        to_be_removed = [t for t in self.graph if isinstance(t[2], rdflibLiteral)]
        for t in to_be_removed:
            (subj, pred, obj) = t
            # Test the validity of the datatype
            if obj.datatype:
                converter = AltXSDToPYTHON.get(obj.datatype, identity)
                try:
                    converter(str(obj))
                except ValueError:
                    closure.add_error("Lexical value of the literal '%s' does not match its datatype (%s)" %
                                      (str(obj), obj.datatype))

            # Check if a BNode has already been associated with that literal
            obj_st = _LiteralStructure(obj)
            items = (b for l, b in self.lit_to_bnode.items() if l == obj_st)
            l_bnode = next(items, None)
            if l_bnode:
                to_be_added.add((subj, pred, l_bnode))
            else:
                _add_bnode(subj, pred, obj_st)
                # Furthermore: a plain literal should be identified with a corresponding xsd:string and vice versa, 
                # cf, RDFS Semantics document
                if obj_st.dt is None and obj_st.lang is None:
                    newLit = rdflibLiteral(obj_st.lex, datatype=ns_xsd["string"])
                    new_obj_st = _LiteralStructure(newLit)
                    new_obj_st.dt = ns_xsd["string"]
                    _add_bnode(subj, pred, new_obj_st)
                elif obj_st.dt == ns_xsd["string"]:
                    newLit = rdflibLiteral(obj_st.lex, datatype=None)
                    new_obj_st = _LiteralStructure(newLit)
                    # new_obj_st = _LiteralStructure(obj) # Was this the correct one, or was this an old bug?
                    new_obj_st.dt = None
                    _add_bnode(subj, pred, new_obj_st)
        
        # Do the real modifications
        self._massageGraph(to_be_removed, to_be_added)
        
    def restore(self):
        """
        This method is to be invoked at the end of the forward chain processing. It restores literals (whenever
        possible) to their original self...
        """    
        to_be_removed = set()
        to_be_added = set()
        for t in self.graph:
            (subj, pred, obj) = t
            # The two cases, namely when the literal appears in subject or object positions, should be treated
            # differently
            if subj in self.bnode_to_lit:
                # well... there may be to cases here: either this is the original tuple stating that
                # this bnode is a literal, or it is the result of an inference. In both cases, the tuple must
                # be removed from the result without any further action
                to_be_removed.add(t)
            elif obj in self.bnode_to_lit:
                # This is where the exchange should take place: put back the real literal into the graph, removing the
                # proxy one
                to_be_removed.add(t)
                # This is an additional thing due to the latest change of literal handling in RDF concepts.
                # If a literal is an xsd:string then a plain literal is put in its place for the purpose of
                # serialization...
                lit = self.bnode_to_lit[obj].lit
                if lit.datatype is not None and lit.datatype == ns_xsd["string"]:
                    lit = rdflibLiteral(str(lit))
                to_be_added.add((subj, pred, lit))
                
        # Do the real modifications
        self._massageGraph(to_be_removed, to_be_added)
        
    def _massageGraph(self, to_be_removed, to_be_added):
        """
        Perform the removal and addition actions on the graph
        @param to_be_removed: list of tuples to be removed
        @param to_be_added: list of tuples to be added
        """
        for t in to_be_removed:
            self.graph.remove(t)
        for t in to_be_added:
            self.graph.add(t)


def add_bnode(proxy, to_be_added, subj, pred, obj):
    """
    Create and add BNode for a literal and register the new object within
    the :class:`.LiteralProxies` class data structures.
    """
    bn = BNode()
    # store this in the internal administration
    proxy.lit_to_bnode[obj] = bn
    proxy.bnode_to_lit[bn] = obj
    # modify the graph
    to_be_added.add((subj, pred, bn))
    to_be_added.add((bn, rdf_type, Literal))



"""
RDF(S) terms. Note that the set of terms is *complete*, i.e., it includes *all* OWL 2 terms, regardless of whether the
term is used in OWL 2 RL or not.

**Requires**: `RDFLib`_, 4.0.0 and higher.

.. _RDFLib: https://github.com/RDFLib/rdflib

**License**: This software is available for use under the `W3C Software License`_.

.. _W3C Software License: http://www.w3.org/Consortium/Legal/2002/copyright-software-20021231

**Organization**: `World Wide Web Consortium`_

.. _World Wide Web Consortium: http://www.w3.org

**Author**: `Ivan Herman`_

.. _Ivan Herman: http://www.w3.org/People/Ivan/
"""

import rdflib
from rdflib				import Namespace

RDFNS = Namespace("http://www.w3.org/1999/02/22-rdf-syntax-ns#")
RDFSNS = Namespace("http://www.w3.org/2000/01/rdf-schema#")

# RDF Classes
Seq = RDFNS["Seq"]
Bag = RDFNS["Bag"]
Alt = RDFNS["Alt"]
Statement = RDFNS["Statement"]
Property = RDFNS["Property"]
XMLLiteral = RDFNS["XMLLiteral"]
HTMLLiteral = RDFNS["HTML"]
LangString = RDFNS["LangString"]
List = RDFNS["List"]

# RDF Properties
rdf_subject = RDFNS["subject"]
rdf_predicate = RDFNS["predicate"]
rdf_object = RDFNS["object"]
rdf_type = RDFNS["type"]
value = RDFNS["value"]
first = RDFNS["first"]
rest = RDFNS["rest"]
# and _n where n is a non-negative integer

# RDF Resources
nil = RDFNS["nil"]

Resource = RDFSNS["Resource"]
Class = RDFSNS["Class"]
subClassOf = RDFSNS["subClassOf"]
subPropertyOf = RDFSNS["subPropertyOf"]
comment = RDFSNS["comment"]
label = RDFSNS["label"]
rdfs_domain = RDFSNS["domain"]
rdfs_range = RDFSNS["range"]
seeAlso = RDFSNS["seeAlso"]
isDefinedBy = RDFSNS["isDefinedBy"]
Literal = RDFSNS["Literal"]
Container = RDFSNS["Container"]
ContainerMembershipProperty = RDFSNS["ContainerMembershipProperty"]
member = RDFSNS["member"]
Datatype = RDFSNS["Datatype"]
